Screen {
  align: center middle;
  layers: main footer;
}

Request {
  layout: vertical;
  background: $boost;
  padding: 1;
  margin: 1 0;
  min-height: 100%;
  height: auto;
}

#id-container {
  layout: horizontal;
  height: 1;
}

#id {
  width: auto;
  margin-right: 1;
}

#date {
  width: auto;
  margin-right: 1;
}

#request-engine-container {
  background: red;
  background: $background-lighten-2;
}

#request-temperature-container,
#request-prompt-container,
#request-instruction-container,
#request-input-container {
  height: auto;
  background: $background-lighten-2;
  border-bottom: solid peachpuff;
}
#results-header,
#request-header {
  background: $primary-darken-2;
  align: center middle;
  text-align: center;
  margin-top: 1;
}
#response-text-container {
  height: auto;
  max-height: 40vh;
  min-height: 4;
  background: $background-lighten-2;
  border-left: wide $primary-darken-2;
  border-right: wide $primary-darken-2;
  border-bottom: wide $primary-darken-2;
}
#requests {
  background: $boost;
  border-left: wide $primary-darken-2;
  border-right: wide $primary-darken-2;
  border-bottom: wide $primary-darken-2;
  /* padding-left: 1; */
  margin-top: 0;
  height: auto;
  max-height: 40vh;
}

.hidden {
  visibility: hidden;
  height: 0;
  border: none;
  margin: 0;
  padding: 0;
  width: 1;
}

PromptSidebar {
  height: 100vh;
  width: auto;
  min-width: 20;
  background: $secondary-background-darken-2;
  dock: left;
  margin-right: 1;
}

PromptSidebar Button {
  height: auto;
  width: 18;
}
Footer {
  layer: footer;
}
